/* HtmlTableMiner.rex
 extracts text within an HTML file that are not within tags
 and are enclosed within <table> and </table>

 the output contains tabs between consecutive column values.

 ..when the end of a table is encountered the program writes
 an empty line, a line containing three dots, and another
 empty line.

 usage
  r4 HtmlTableMiner < inFile.htm > outFile.txt
*/

inComment = 0

inTag = 0

textOut = ''

emit = 0

anotherCol = 0

tag = ''

end_marker_pending = 0

do lno=1 for lines()

  lin = linein()

  if lin = '' then iterate /* ignore empty lines */

  /* trace ?r */

  htmlLine = lin

  do inner=1 while lin <> ''
    
    if inComment then do
      if pos( '-->', lin ) > 0 then
        inComment = 0
      parse var lin . '-->' lin
      iterate inner
      end /* inComment */
    
    if inTag then do
      if pos( '>', lin ) > 0 then do
        parse var lin tag '>' lin
        inTag = 0
        end /* pos( '>', lin ) > 0 */
      else
        leave inner
      end /* inTag */

    if pos( '<', lin ) = 0 then do
      if emit > 0 then
        call AddSegment ( translate( lin, '  ', '0a0d'x ) )
      leave inner
      end /* pos( '<', lin ) = 0 */

    parse var lin before '<' lin

    if before <> '' then do
      if emit > 0 then
        call AddSegment ( translate( before, '  ', '0a0d'x ) )
      end

    if left( lin, 3 ) = '!--' then
      inComment = 1

    else if lin <> '' then do
           
      if pos( '>', lin ) = 0 then
        inTag = 1
        
      parse var lin tag '>' lin

      tag = translate( word( tag, 1 ) )

      if tag = 'TABLE' then do
        emit = emit + 1
        anotherCol = 0
        end /* tag = 'TABLE' */

      else if tag = '/TABLE' then do
        textOut = textOut || '0a0d'x || '...' || '0a0d'x
        emit = emit - 1
        anotherCol = 0
        end /* tag = '/TABLE' */

      else if wordpos( tag, 'TD TH' ) > 0 then do
        if anotherCol then do
          textOut = textOut || '09'x
          anotherCol = 0
          end /* anotherCol */
        end /* wordpos( tag, 'TD TH' ) > 0 */

      else if wordpos( tag, '/TD /TH' ) > 0  then
          anotherCol = 1

      else if tag = '/TR' then do
        anotherCol = 0
        say textOut
        textOut = ''
        end /* tag = '/TR' */
        
      end /* lin <> '' */
      
    end inner /* while lin <> '' */

  tag = ''

  end lno

say textOut

exit 0

AddSegment : procedure expose htmlLine textOut tag

  parse arg text
  
  if text = '' then
    return
  
  text = strip( text )
  
  if pos( '&', text ) = 0 then do
    call Add text
    return
    end /* pos( '&', text ) = 0 */

  do while text <> ''

    if pos( '&', text ) = 0 then do
      call Add text
      return
      end /* pos( '&', text ) = 0 */

    parse var text before '&' ident ';' rest
	  
    if before <> '' then
      call Add before

    if ident = '' then do
      call lineout !, 'Poorly formed HTML text encountered. The invalid line is:'
      call lineout !, '  'htmlLine
      exit 2
      end /* ident = '' */

    text = rest

    select
      when ident = 'nbsp' then
        call Add ' '
      when ident = 'lt' then
        call Add '<'
      when ident = 'gt' then
        call Add '>'
      when ident = 'amp' then
        call Add '&'
      when ident = '#153' then
        call Add '(TM)'
      when ident = 'reg' then
        call Add '(R)'
      when ident = 'copy' then
        call Add '(C)'
      otherwise
        call Add '&'ident';'
      end /* select */
      
    end /* while text <> '' */
    
  return /* AddSegment */

Add : procedure expose textOut tag
  if wordpos( tag, '/U /S' ) = 0 ,
   & textOut <> '' ,
   & pos( right( textOut, 1 ), '090d'x ) = 0 ,
   & right( textOut, 1 ) <> ' ' then
    textOut = textOut' '
  
  textOut = textOut || arg( 1 )
  return
