/* preprocess.rex

 usage:
  r4 symbol* < inFile > outFile
  
 example:
  r4 _debug < inFile.java > outFile.java
  
 the PREPROCESS program is a s-i-m-p-l-e
 source file preprocessor.
 
 A series of symbols can be 'defined' on the
 command line.
 
 Other symbols can be defined in the source as follows:
  #define _other
  
 Then, a segment can be optionally emitted as follows:
  # ifdef _debug
  trace 'I'
  # endif
  
 Other files can be added to the output stream as follows:
  # include "otherFile"
 Or, 
  # include 'otherFile'
 Or, 
  # include otherFile
  
 ---
  
 CAVEAT:
  
  This version does NOT do #symbol replacement.
  
   For example:
    #define CRLF '0d0a'x
  
   In the following:
    say 'Is this two lines' || CRLF || 'Or just one ?'

   The generated line will be:  
    say 'Is this two lines' || CRLF || 'Or just one ?'

   Instead of:
    say 'Is this two lines' || '0d0a'x || 'Or just one ?'
*/

if lines() = 0 then do
  call lineout !, 'The input file is empty.'
  call lineout !, ''
  signal usagemsg
  end

symbols = arg( 1 )

symbol. = 0

do while symbols <> ''
  parse var symbols symbol symbols
  symbol.symbol = 1
  end
  
emit = 1
  
do lno=1 for lines()
  
  call processLine ''
  
  end

exit 0
  
include : procedure expose lin emit symbol.
  parse arg lno, fil
  
  if fil = '' then do
    call lineout !, 'Empty file name encountered on line #' lno
    call lineout !, '  'lin
    exit 1
    end
    
  call lineout !, 'Including:' fil    
  
  do lno=1 for lines( fil )
    call processLine fil
    end
    
  call lineout fil /* close the included file */
  
  return
  
processLine :  
  
  parse value linein( arg(1) ) with ,
    lin 1 orderch +1 order symbol rest 1 . +1 . inclfile
  
  if orderch <> '#' then do
    if emit then
      say strip( lin, 'T' )
      
    return
    end
    
  order = strip( order )
    
  select
    
    when order = 'define' then
      symbol.symbol = rest || copies( '1', rest = '' )
      
    when order = 'ifdef' then do
      push emit
      emit = ( symbol.symbol <> '0' )
      end
      
    when order = 'ifndef' then do
      push emit
      emit = ( symbol.symbol = '0' )
      end
      
    when order = 'else' then
      emit = \ emit
      
    when order = 'endif' then
      pull emit
      
    when order = 'include' then
      call include lno, strip( translate( inclfile, '  ', "'" || '"' ) )
      
    otherwise
      call lineout !, 'Unrecognized preprocessor order in line #' lno
      call lineout !, '  'lin
      exit 1
    
    end
    
  return

usagemsg :
  call lineout !, 'Usage:'
  call lineout !, ''
  call lineout !, '  r4 symbol* < inFile > outFile'  
  
